@@ -43,8 +43,8 @@ my $debug       = $opts{d} || undef;
 my $community   = $opts{c} || 'public';
 my $version     = $opts{v} || '2';
 my $nonblocking = $opts{B} || 0;
-my $timeout     = $opts{t} || 5;
-my $retries     = $opts{t} || 0;
+my $timeout     = $opts{t} || 3;
+my $retries     = $opts{t} || 1;
 
 my $from_stdin = $opts{i} || undef;
 
@@ -74,22 +74,17 @@ foreach my $agent ( sort @agents ) {
   $session->init_mixins;
   push @sessions, $session;
 }
+
 snmp_dispatcher();
 
-# warn on errors during initialization
-foreach my $session (@sessions) {
-  if ( $session->errors ) {
-    foreach my $error ( $session->errors ) {
-      warn $session->hostname . ": $error\n";
-    }
-  }
-}
-
-# remove sessions with errors from the sessions list
-@sessions = grep { not $_->errors(1) } @sessions;
+# check for init errors
+$_->init_ok foreach @sessions;
 
 # remove sessions with error from the sessions list
-@sessions = grep { warn $_->error if $_->error; not $_->error } @sessions;
+@sessions = grep {
+  if ( $_->errors ) { warn scalar $_->errors, "\n"; undef }
+  else              { 1 }
+} @sessions;
 
 foreach my $session ( sort { $a->hostname cmp $b->hostname } @sessions ) {
   print_if($session);
@@ -147,7 +142,7 @@ Karl Gaissmaier, karl.gaissmaier (at) uni-ulm.de
 
 =head1 COPYRIGHT
 
-Copyright (C) 2008 by Karl Gaissmaier
+Copyright (C) 2008-2015 by Karl Gaissmaier
 
 This program is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
