@@ -14,7 +14,7 @@ my $prefix = __PACKAGE__;
 # this module import config
 #
 use Carp ();
-use Net::SNMP::Mixin::Util qw/idx2val push_error/;
+use Net::SNMP::Mixin::Util qw/idx2val push_error get_init_slot/;
 
 #
 # this module export config
@@ -48,11 +48,11 @@ Net::SNMP::Mixin::IfInfo - mixin class for interface related infos
 
 =head1 VERSION
 
-Version 0.05
+Version 0.07
 
 =cut
 
-our $VERSION = '0.05';
+our $VERSION = '0.07';
 
 =head1 SYNOPSIS
 
@@ -64,6 +64,7 @@ our $VERSION = '0.05';
   $session->mixer('Net::SNMP::Mixin::IfInfo');
   $session->init_mixins;
   snmp_dispatcher();
+  $session->init_ok();
   die $session->errors if $session->errors;
 
   my $if_entries = $session->get_if_entries;
@@ -111,8 +112,10 @@ Returns parts ot the ifTable and ifXTable as a hash reference. The key is the co
 
 sub get_if_entries {
   my $session = shift;
-  Carp::croak "'$prefix' not initialized,"
-    unless $session->{$prefix}{__initialized};
+  my $agent   = $session->hostname;
+
+  Carp::croak "$agent: '$prefix' not initialized,"
+    unless $session->init_ok($prefix);
 
   # stash for return values
   my $result = {};
@@ -149,11 +152,23 @@ Fetch basic interface related snmp values from the host. Don't call this method
 
 =cut
 
-sub _init {
-  my ($session, $reload) = @_;
+#
+# due to the asynchron nature, we don't know what init job is really the last, we decrement
+# the value after each callback
+#
+use constant THIS_INIT_JOBS => 2;
 
-  die "$prefix already initalized and reload not forced.\n"
-    if $session->{$prefix}{__initialized} && not $reload;
+sub _init {
+  my ( $session, $reload ) = @_;
+  my $agent = $session->hostname;
+
+  die "$agent: $prefix already initialized and reload not forced.\n"
+    if exists get_init_slot($session)->{$prefix}
+      && get_init_slot($session)->{$prefix} == 0
+      && not $reload;
+
+  # set number of async init jobs for proper initialization
+  get_init_slot($session)->{$prefix} = THIS_INIT_JOBS;
 
   # populate the object with needed mib values
   #
@@ -239,7 +254,10 @@ sub _if_table_entries_cb {
   $session->{$prefix}{ifInfo}{ifOperStatus} =
     idx2val( $vbl, IF_OPER_STATUS );
 
-  $session->{$prefix}{__initialized}++;
+  # this init job is finished
+  get_init_slot($session)->{$prefix}--;
+
+  return 1;
 }
 
 =head2 B<< _fetch_if_x_table_entries($session) >>
@@ -301,7 +319,10 @@ sub _if_x_table_entries_cb {
 
   $session->{$prefix}{ifInfo}{ifAlias} = idx2val( $vbl, IF_X_ALIAS );
 
-  $session->{$prefix}{__initialized}++;
+  # this init job is finished
+  get_init_slot($session)->{$prefix}--;
+
+  return 1;
 }
 
 =head1 SEE ALSO
@@ -326,7 +347,7 @@ Karl Gaissmaier <karl.gaissmaier at uni-ulm.de>
 
 =head1 COPYRIGHT & LICENSE
 
-Copyright 2008 Karl Gaissmaier, all rights reserved.
+Copyright 2008-2016 Karl Gaissmaier, all rights reserved.
 
 This program is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
